# flake8: noqa
import os

from celery.schedules import crontab

from django.contrib import messages

PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)
SITE_URL = "http://localhost:8000"
CSRF_TRUSTED_ORIGINS = [SITE_URL]
ESI_USER_CONTACT_EMAIL = "devnull@example.com"

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.humanize",
    "django_celery_beat",
    "django_celery_results",
    "bootstrapform",
    "sortedm2m",
    "esi",
    "eve_sde",
    "allianceauth",
    "allianceauth.authentication",
    "allianceauth.services",
    "allianceauth.eveonline",
    "allianceauth.groupmanagement",
    "allianceauth.notifications",
    "allianceauth.thirdparty.navhelper",
    "miningtaxes",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.locale.LocaleMiddleware",
]

ROOT_URLCONF = "testauth.urls"
WSGI_APPLICATION = "testauth.wsgi.application"

SECRET_KEY = "testauth-aa4-secret-key"
DEBUG = False
ALLOWED_HOSTS = ["*"]
SITE_NAME = "testauth"
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_TZ = True
STATIC_URL = "/static/"
STATIC_ROOT = os.path.join(BASE_DIR, "static")
STATICFILES_DIRS = []
LOCALE_PATHS = (os.path.join(BASE_DIR, "locale/"),)
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
SILENCED_SYSTEM_CHECKS = ["allianceauth.checks.B003", "allianceauth.checks.B004"]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(PROJECT_DIR, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "allianceauth.context_processors.auth_settings",
            ],
        },
    },
]

AUTHENTICATION_BACKENDS = [
    "allianceauth.authentication.backends.StateBackend",
    "django.contrib.auth.backends.ModelBackend",
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": str(os.path.join(BASE_DIR, "alliance_auth.sqlite3")),
    },
}

CACHES = {
    "default": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/15",
        "OPTIONS": {
            "IGNORE_EXCEPTIONS": True,
        },
    }
}

MESSAGE_TAGS = {messages.ERROR: "danger"}

LOGIN_URL = "auth_login_user"
LOGIN_REDIRECT_URL = "authentication:dashboard"
LOGOUT_REDIRECT_URL = "authentication:dashboard"
LOGIN_TOKEN_SCOPES = ["publicData"]
ACCOUNT_ACTIVATION_DAYS = 1

ESI_API_URL = "https://esi.evetech.net/"
ESI_SSO_CLIENT_ID = "dummy"
ESI_SSO_CLIENT_SECRET = "dummy"
ESI_SSO_CALLBACK_URL = "http://localhost:8000"

REGISTRATION_VERIFY_EMAIL = False
EMAIL_HOST = ""
EMAIL_PORT = 587
EMAIL_HOST_USER = ""
EMAIL_HOST_PASSWORD = ""
EMAIL_USE_TLS = True
DEFAULT_FROM_EMAIL = ""

CELERY_BROKER_URL = "memory://"
CELERY_RESULT_BACKEND = "django-db"
CELERY_TASK_ALWAYS_EAGER = True
CELERY_TASK_EAGER_PROPAGATES = True
CELERY_BEAT_SCHEDULER = "django_celery_beat.schedulers.DatabaseScheduler"
broker_transport_options = {
    "priority_steps": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
    "queue_order_strategy": "priority",
}
CELERY_BROKER_TRANSPORT_OPTIONS = {
    "priority_steps": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
    "queue_order_strategy": "priority",
}
broker_connection_retry_on_startup = True
CELERY_BROKER_CONNECTION_RETRY_ON_STARTUP = True
CELERY_BEAT_SCHEDULE = {
    "esi_cleanup_callbackredirect": {
        "task": "esi.tasks.cleanup_callbackredirect",
        "schedule": crontab(minute=0, hour="*/4"),
    },
    "esi_cleanup_token": {
        "task": "esi.tasks.cleanup_token",
        "schedule": crontab(minute=0, hour=0),
    },
    "run_model_update": {
        "task": "allianceauth.eveonline.tasks.run_model_update",
        "schedule": crontab(minute=0, hour="*/6"),
    },
    "check_all_character_ownership": {
        "task": "allianceauth.authentication.tasks.check_all_character_ownership",
        "schedule": crontab(minute=0, hour="*/4"),
    },
}

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "verbose": {
            "format": "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s",
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
    },
    "handlers": {
        "console": {
            "level": "CRITICAL",
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        },
    },
    "loggers": {
        "allianceauth": {"handlers": ["console"], "level": "DEBUG"},
        "django": {"handlers": ["console"], "level": "ERROR"},
        "esi": {"handlers": ["console"], "level": "DEBUG"},
    },
}



from django.core.management.base import BaseCommand

from ... import tasks
from ...models import AdminCharacter, Character


class Command(BaseCommand):
    help = "Runs daily update manually"

    def handle(self, *args, **options):
        print("Beginning price update")
        tasks.update_all_prices()

        print("Beginning admin character updates")
        characters = AdminCharacter.objects.all()
        for character in characters:
            tasks.update_admin_character(character_pk=character.id)

        print("Running character update in parallel")
        characters = Character.objects.all()
        for character in characters:
            tasks.update_character(character_pk=character.id)

        tasks.precalcs([1, 1])



from esi.openapi_clients import ESIClientProvider

from . import (
    __app_name_verbose__,
    __esi_compatibility_date__,
    __git_repo_url__,
    __version__,
)

esi = ESIClientProvider(
    compatibility_date=__esi_compatibility_date__,
    ua_appname=__app_name_verbose__,
    ua_version=__version__,
    ua_url=__git_repo_url__,
    operations=[
        "GetCorporationCorporationIdMiningObservers",
        "GetUniverseStructuresStructureId",
        "GetCharactersCharacterIdMining",
        "GetCorporationsCorporationIdWalletsDivisionJournal",
        "GetCorporationCorporationIdMiningObserversObserverId",
    ],
)



import datetime as dt

from dateutil.relativedelta import relativedelta

from django.utils.timezone import now

from allianceauth.tests.auth_utils import AuthUtils
from app_utils.testing import NoSocketsTestCase, add_new_token

from ..models import (
    AdminCharacter,
    AdminMiningCorpLedgerEntry,
    Character,
    OrePrices,
    Settings,
    Stats,
)
from .testdata.load_entities import load_entities
from .testdata.load_eve_sde import load_eve_sde
from .utils import (
    add_auth_character_to_user,
    add_miningtaxes_character_to_user,
    create_user_from_evecharacter_with_access,
)


class MiningTaxesBaseTestCase(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_eve_sde()
        load_entities()

    def setUp(self):
        super().setUp()
        self._reset_singletons()
        self.user, self.ownership = create_user_from_evecharacter_with_access(1001)
        self.user = AuthUtils.add_permission_to_user_by_name(
            "miningtaxes.admin_access", self.user
        )
        self.user = AuthUtils.add_permission_to_user_by_name(
            "miningtaxes.auditor_access", self.user
        )
        self.main_character = Character.objects.update_or_create(
            eve_character=self.ownership.character
        )[0]
        self.alt_character = add_miningtaxes_character_to_user(self.user, 1002)
        self.admin_character = AdminCharacter.objects.update_or_create(
            eve_character=self.ownership.character
        )[0]
        self.other_user, self.other_ownership = (
            create_user_from_evecharacter_with_access(1101)
        )
        self.other_character = Character.objects.update_or_create(
            eve_character=self.other_ownership.character
        )[0]
        self.unregistered_ownership = add_auth_character_to_user(self.user, 1102)
        self.prev_month = (now().date() + relativedelta(months=-1)).replace(day=15)
        self.current_month = now().date().replace(day=min(10, now().date().day))
        self.seed_prices()
        self.seed_ledgers_and_admin_data()
        add_new_token(self.user, self.ownership.character, Character.get_esi_scopes())
        add_new_token(
            self.user, self.ownership.character, AdminCharacter.get_esi_scopes()
        )

    def _reset_singletons(self):
        settings = Settings.load()
        settings.phrase = ""
        settings.interest_rate = 5.0
        settings.save()
        stats = Stats.load()
        for field in [
            "admin_char_json",
            "admin_main_json",
            "ore_prices_json",
            "admin_mining_by_sys_json",
            "admin_tax_revenue_json",
            "admin_month_json",
            "admin_corp_ledger",
            "admin_corp_mining_history",
            "leaderboards",
            "admin_get_all_activity_json",
            "curmonth_leadergraph",
            "user_mining_ledger_90day",
        ]:
            setattr(stats, field, None)
        stats.save()

    def seed_prices(self):
        for type_id, buy, sell in [
            (45511, 10, 100),
            (62586, 5, 50),
            (1230, 2, 20),
            (16635, 20, 200),
        ]:
            ore, _ = OrePrices.objects.update_or_create(
                eve_type_id=type_id,
                defaults={"buy": buy, "sell": sell, "updated": now()},
            )
            ore.calc_prices()

    def add_ledger_entry(
        self, character, date, quantity, eve_type_id=45511, system_id=30002537
    ):
        entry = character.mining_ledger.create(
            date=date,
            quantity=quantity,
            eve_type_id=eve_type_id,
            eve_solar_system_id=system_id,
        )
        entry.calc_prices()
        return entry

    def seed_ledgers_and_admin_data(self):
        self.add_ledger_entry(self.main_character, self.prev_month, 10, 45511, 30002537)
        self.add_ledger_entry(
            self.main_character, self.current_month, 5, 1230, 30000142
        )
        self.add_ledger_entry(self.alt_character, self.prev_month, 7, 45511, 30000142)
        self.add_ledger_entry(
            self.other_character, self.prev_month, 11, 45511, 31000005
        )
        self.main_character.give_credit(25, "paid")
        self.alt_character.give_credit(5, "credit")

        AdminMiningCorpLedgerEntry.objects.create(
            character=self.admin_character,
            date=now() - dt.timedelta(days=20),
            taxed_id=self.main_character.eve_character.character_id,
            amount=150.0,
            reason="moon tax",
        )
        AdminMiningCorpLedgerEntry.objects.create(
            character=self.admin_character,
            date=now() - dt.timedelta(days=10),
            taxed_id=1102,
            amount=75.0,
            reason="moon tax",
        )
        AdminMiningCorpLedgerEntry.objects.create(
            character=self.admin_character,
            date=now() - dt.timedelta(days=5),
            taxed_id=9999,
            amount=20.0,
            reason="other",
        )

        observer = self.admin_character.mining_obs.create(
            obs_id=123456789,
            obs_type="structure",
            name="Amamake Test",
            sys_name="Amamake",
        )
        observer.mining_log.create(
            date=self.current_month,
            miner_id=self.main_character.eve_character.character_id,
            eve_type_id=45511,
            quantity=100,
            observer_type="structure",
            eve_solar_system_id=30002537,
        )
        observer.mining_log.create(
            date=self.current_month,
            miner_id=1102,
            eve_type_id=45511,
            quantity=50,
            observer_type="structure",
            eve_solar_system_id=30002537,
        )
        observer.mining_log.create(
            date=self.current_month,
            miner_id=9999,
            eve_type_id=45511,
            quantity=20,
            observer_type="structure",
            eve_solar_system_id=30002537,
        )



# Generated by Django 4.0.7 on 2023-05-28 01:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0007_character_life_credits_character_life_taxes"),
    ]

    operations = [
        migrations.AddField(
            model_name="character",
            name="monthly_credits_json",
            field=models.JSONField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="character",
            name="monthly_mining_json",
            field=models.JSONField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="character",
            name="monthly_taxes_json",
            field=models.JSONField(default=None, null=True),
        ),
    ]



from unittest.mock import patch

from django.contrib.auth.models import Permission
from django.utils.timezone import now
from esi.errors import TokenError

from allianceauth.eveonline.models import EveCharacter

from .. import tasks
from ..models import (
    AdminCharacter,
    AdminMiningCorpLedgerEntry,
    Character,
    OrePrices,
    Settings,
    Stats,
)
from .base import MiningTaxesBaseTestCase


class TestTasks(MiningTaxesBaseTestCase):
    def test_calctaxes_and_get_user(self):
        self.assertIn(self.user, tasks.calctaxes())
        self.assertEqual(
            tasks.get_user(self.main_character.eve_character.character_id),
            self.main_character,
        )
        self.assertIsNone(tasks.get_user(9999))

    @patch("miningtaxes.tasks.notify")
    @patch("miningtaxes.tasks.calctaxes")
    def test_notify_tasks(self, mock_calctaxes, mock_notify):
        mock_calctaxes.return_value = {self.user: [100.0, 0.0, self.main_character]}
        tasks.notify_taxes_due.run()
        tasks.notify_second_taxes_due.run()
        self.assertEqual(mock_notify.call_count, 2)

    @patch("miningtaxes.tasks.notify")
    @patch("miningtaxes.tasks.Stats.load")
    def test_notify_current_threshold(self, mock_load, mock_notify):
        mock_load.return_value.get_admin_main_json.return_value = [
            {"balance": 2000000000, "user": self.user.pk},
            {"balance": 1, "user": 9999},
        ]
        tasks.notify_current_taxes_threshold.run()
        mock_notify.assert_called_once()

    @patch("miningtaxes.tasks.notify")
    @patch("miningtaxes.tasks.calctaxes")
    def test_apply_interest(self, mock_calctaxes, mock_notify):
        settings = Settings.load()
        settings.interest_rate = 10
        settings.save()
        mock_calctaxes.return_value = {self.user: [100.0, 0.0, self.main_character]}
        tasks.apply_interest.run()
        self.assertTrue(
            self.main_character.tax_credits.filter(credit_type="interest").exists()
        )
        mock_notify.assert_called_once()

    @patch("miningtaxes.tasks.requests.get")
    def test_valid_janice_api_key(self, mock_get):
        mock_get.return_value.json.return_value = {}
        self.assertTrue(tasks.valid_janice_api_key())
        mock_get.return_value.json.return_value = {"status": "bad"}
        self.assertFalse(tasks.valid_janice_api_key())

    @patch("miningtaxes.tasks.requests.post")
    @patch("miningtaxes.tasks.requests.get")
    def test_get_bulk_prices(self, mock_get, mock_post):
        with patch("miningtaxes.tasks.MININGTAXES_PRICE_METHOD", "Fuzzwork"):
            mock_get.return_value.json.return_value = {
                "45511": {"buy": {"max": "1"}, "sell": {"min": "2"}}
            }
            self.assertIn("45511", tasks.get_bulk_prices([45511]))

        with patch("miningtaxes.tasks.MININGTAXES_PRICE_METHOD", "Janice"):
            mock_post.return_value.json.return_value = [
                {
                    "itemType": {"eid": 45511},
                    tasks.MININGTAXES_PRICE_JANICE_TIMING: {
                        tasks.MININGTAXES_PRICE_JANICE_BUY: 3,
                        tasks.MININGTAXES_PRICE_JANICE_SELL: 4,
                    },
                }
            ]
            self.assertEqual(tasks.get_bulk_prices([45511])["45511"]["buy"]["max"], "3")

    @patch("miningtaxes.tasks.get_bulk_prices")
    def test_update_all_prices(self, mock_bulk):
        def fake_bulk(type_ids):
            return {
                str(tid): {"buy": {"max": "11"}, "sell": {"min": "22"}}
                for tid in type_ids
            }

        mock_bulk.side_effect = fake_bulk
        OrePrices.objects.all().delete()
        tasks.update_all_prices.run(force=[45511])
        self.assertTrue(OrePrices.objects.filter(eve_type_id=45511).exists())

    def test_update_all_prices_non_fuzzwork_branches(self):
        with patch("miningtaxes.tasks.MININGTAXES_PRICE_METHOD", "Invalid"):
            self.assertIsNone(tasks.update_all_prices.run(force=[]))
        with patch("miningtaxes.tasks.MININGTAXES_PRICE_METHOD", "Janice"), patch(
            "miningtaxes.tasks.valid_janice_api_key", return_value=False
        ):
            self.assertIsNone(tasks.update_all_prices.run(force=[]))

    @patch("miningtaxes.tasks.add_corp_moon_taxes")
    @patch("miningtaxes.tasks.Stats.load")
    def test_precalcs(self, mock_stats_load, mock_add_corp):
        stats = Stats.load()
        mock_stats_load.return_value = stats
        with patch.object(stats, "precalc_all") as mock_stats_precalc, patch.object(
            Character, "precalc_all"
        ) as mock_char_precalc:
            tasks.precalcs.run([True, False])
            self.assertTrue(mock_char_precalc.called)
            mock_stats_precalc.assert_called_once()
            mock_add_corp.assert_called_once()

    @patch("miningtaxes.tasks.update_character")
    @patch("esi.models.Token.get_token")
    def test_auto_add_chars(self, mock_get_token, mock_update_character):
        tracked = EveCharacter.objects.create(
            character_id=1200,
            character_name="Tim Drake",
            corporation_id=self.admin_character.eve_character.corporation_id,
            corporation_name=self.admin_character.eve_character.corporation_name,
            corporation_ticker=self.admin_character.eve_character.corporation_ticker,
            alliance_id=self.admin_character.eve_character.alliance_id,
            alliance_name=self.admin_character.eve_character.alliance_name,
            alliance_ticker=self.admin_character.eve_character.alliance_ticker,
        )
        tracked_ownership = self.user.character_ownerships.create(character=tracked)
        self.user.profile.main_character = self.ownership.character
        self.user.profile.save()
        mock_get_token.return_value = object()

        tasks.auto_add_chars.run()

        self.assertTrue(mock_update_character.called)
        self.assertTrue(
            Character.objects.filter(eve_character=tracked_ownership.character).exists()
        )

    @patch("miningtaxes.tasks.chord")
    @patch("miningtaxes.tasks.update_character")
    @patch("miningtaxes.tasks.update_admin_character")
    @patch("miningtaxes.tasks.update_all_prices")
    @patch("miningtaxes.tasks.precalcs")
    def test_update_daily(
        self,
        mock_precalcs,
        mock_prices,
        mock_update_admin,
        mock_update_character,
        mock_chord,
    ):
        mock_update_character.s.side_effect = lambda **kwargs: kwargs
        mock_precalcs.s.return_value = "precalc-sig"
        mock_chord.return_value = lambda sig: None
        tasks.update_daily.run()
        mock_prices.assert_called_once()
        self.assertTrue(mock_update_admin.called)
        mock_chord.assert_called_once()

    def test_add_credit_helpers(self):
        tasks.add_tax_credits()
        self.assertTrue(self.main_character.tax_credits.filter(credit=150.0).exists())

        AdminMiningCorpLedgerEntry.objects.create(
            character=self.admin_character,
            date=now(),
            taxed_id=self.main_character.eve_character.character_id,
            amount=12.0,
            reason="moon",
        )
        settings = Settings.load()
        settings.phrase = "moon"
        settings.save()
        tasks.add_tax_credits_by_char(self.main_character)
        self.assertTrue(self.main_character.tax_credits.filter(credit=12.0).exists())

    def test_add_corp_moon_tax_helpers(self):
        Character.objects.filter(pk=self.main_character.pk).update(life_taxes=0)
        tasks.add_corp_moon_taxes_by_char(self.main_character)
        self.assertTrue(
            self.main_character.mining_ledger.filter(
                eve_type_id=45511, eve_solar_system_id=30002537, quantity=100
            ).exists()
        )
        tasks.add_corp_moon_taxes()

    @patch("miningtaxes.tasks.add_corp_moon_taxes")
    @patch.object(AdminCharacter, "update_all")
    def test_update_admin_character(self, mock_update_all, mock_add_corp):
        self.assertFalse(
            tasks.update_admin_character.run(
                character_pk=self.admin_character.pk, force_update=False
            )
            is True
        )
        tasks.update_admin_character.run(
            character_pk=self.admin_character.pk, force_update=True
        )
        mock_update_all.assert_called()

    @patch("miningtaxes.tasks.users_with_permission")
    @patch("miningtaxes.tasks.notify")
    @patch.object(Character, "update_mining_ledger", side_effect=TokenError())
    def test_update_character_token_error(self, mock_update, mock_notify, mock_users):
        perm = Permission.objects.filter(
            content_type__app_label="miningtaxes", codename="admin_access"
        ).first()
        mock_users.return_value = [self.user]
        self.assertIsNotNone(perm)
        self.assertFalse(
            tasks.update_character.run(
                character_pk=self.main_character.pk, force_update=True
            )
        )
        mock_notify.assert_called_once()

    @patch("miningtaxes.tasks.add_corp_moon_taxes_by_char")
    @patch("miningtaxes.tasks.add_tax_credits_by_char")
    @patch.object(Character, "update_mining_ledger")
    def test_update_character_success(self, mock_update, mock_credits, mock_corp):
        self.assertFalse(
            tasks.update_character.run(
                character_pk=self.main_character.pk, force_update=True
            )
        )
        mock_update.assert_called_once()
        mock_credits.assert_called_once()



default_app_config = "miningtaxes.apps.MiningTaxesConfig"

__version__ = "2.0.1"
__title__ = "Mining Taxes"

__esi_compatibility_date__ = "2025-11-06"  # ESI compatibility date, see https://esi.evetech.net/meta/compatibility-dates

__app_name__ = "miningtaxes"
__app_name_verbose__ = "Alliance Auth Mining Taxes"
__app_name_useragent__ = "AA-Mining-Taxes"

__git_repo_url__ = "https://gitlab.com/arctiru/aa-miningtaxes"



# Shamelessly stolen from Member Audit
from eve_sde.models import ItemType, ItemTypeMaterials

from django.core.exceptions import ObjectDoesNotExist
from django.db import models

from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag

from .. import __title__
from ..app_settings import (
    MININGTAXES_ALWAYS_TAX_REFINED,
    MININGTAXES_REFINED_RATE,
    MININGTAXES_UNKNOWN_TAX_RATE,
)

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def get_tax(eve_type):
    try:
        ore = OrePrices.objects.get(eve_type=eve_type)
        return ore.tax_rate / 100.0
    except OrePrices.DoesNotExist:
        pass
    return MININGTAXES_UNKNOWN_TAX_RATE


def ore_calc_prices(eve_type, q):
    try:
        ore = OrePrices.objects.get(eve_type=eve_type)
        return q * ore.raw_price, q * ore.refined_price, q * ore.taxed_price
    except OrePrices.DoesNotExist:
        pass
    quantity = q
    raw_price = quantity * get_price(eve_type)
    materials = ItemTypeMaterials.objects.filter(item_type_id=eve_type.id)
    refined_price = 0.0
    for mat in materials:
        if mat.quantity is None:
            continue
        q = MININGTAXES_REFINED_RATE * (mat.quantity * quantity) / eve_type.portion_size
        refined_price += q * get_price(mat.material_item_type)
    if refined_price == 0.0:
        refined_price = raw_price
    taxed_value = refined_price
    if not MININGTAXES_ALWAYS_TAX_REFINED and raw_price > taxed_value:
        taxed_value = raw_price
    return raw_price, refined_price, taxed_value


def get_price(eve_type):
    ore = None
    try:
        ore = OrePrices.objects.get(eve_type=eve_type)
    except ObjectDoesNotExist:
        pass
    if ore is None:
        mp = eve_type.base_price
        if mp is None:
            return 0.0
        return mp
    return ore.buy


class OrePrices(models.Model):
    eve_type = models.OneToOneField(
        ItemType,
        on_delete=models.deletion.CASCADE,
        unique=True,
    )
    buy = models.FloatField()
    sell = models.FloatField()
    raw_price = models.FloatField(default=0.0)
    refined_price = models.FloatField(default=0.0)
    taxed_price = models.FloatField(default=0.0)
    tax_rate = models.FloatField(default=10.0)
    updated = models.DateTimeField()

    def calc_prices(self):
        self.raw_price = self.buy
        materials = ItemTypeMaterials.objects.filter(item_type_id=self.eve_type.id)

        self.refined_price = 0.0
        for mat in materials:
            if mat.quantity is None:
                continue
            q = MININGTAXES_REFINED_RATE * mat.quantity / self.eve_type.portion_size
            self.refined_price += q * get_price(mat.material_item_type)
        if self.refined_price == 0.0:
            self.refined_price = self.raw_price
        self.taxed_price = self.refined_price
        if not MININGTAXES_ALWAYS_TAX_REFINED and self.raw_price > self.taxed_price:
            self.taxed_price = self.raw_price
        self.save()
